#include "tg_common.glsl"
 
#ifdef _FRAGMENT_
 
//-----------------------------------------------------------------------------
 
float   Fbm2(vec3 point)
{
    float summ = 0.0;
    float ampl = 1.0;
    for (int i=0; i<cloudsOctaves; ++i)
    {
        summ  += Noise(point) * ampl;
        ampl  *= 0.666; //coverage
        point *= -0.1416; //Noise Scatter Bump
    }
    return summ;
}
 
//-----------------------------------------------------------------------------
 
vec3    Fbm3D2(vec3 point)
{
    vec3  summ = vec3(0.0);
    float ampl = 1.0;
    for (int i=0; i<cloudsOctaves; ++i)
    {
        summ  += NoiseVec3(point) * ampl;
        ampl  *= 0.333; //Noise Scatter Softness
        point *= 3.9761416; //Noise Scatter
    }
    return summ;
}
 
//-----------------------------------------------------------------------------
 
float   HeightMapClouds(vec3 point)
{
    float zone = cos(point.y * twistZones);
    float ang = zone * (twistMagn + 0.05/(twistMagn + 0.1));
    vec3  twistedPoint = point;
    float coverage = cloudsCoverage;
    float weight = 1.0;
    noiseH       = 0.6;
 
    // Compute cyclons
    vec3  cycloneCenter = (tidalLock > 0.0) ? vec3(0.0, 1.0, 0.0) : Cell3NoiseVec((point + Randomize) * cycloneFreq).xyz;
    float cycloneRadius = length(cycloneCenter - point) / ( 0.75 * cycloneSqrtDensity);
    float cycloneAmpl   = -tidalLock * cycloneMagn * sign(cycloneCenter.y);
    if (cycloneRadius < 1.0)
    {
        float dist = 1.0 - cycloneRadius;
        float fi = mix(log(cycloneRadius), pow(dist, 3.0), cycloneRadius);
        twistedPoint = Rotate(cycloneAmpl*fi, cycloneCenter, point);
        weight = saturate(1.0 - cycloneRadius / 1.2);
        weight = (1.0 - weight * weight) * (1.0 + dist);
        coverage = mix(coverage, 1.0, dist);
    }
 
    // Compute the Coriolis effect
    float sina = sin(ang);
    float cosa = cos(ang);
    twistedPoint = vec3(cosa*twistedPoint.x-sina*twistedPoint.z, twistedPoint.y, sina*twistedPoint.x+cosa*twistedPoint.z);
    twistedPoint = twistedPoint * cloudsFreq + Randomize;
 
/*
    // Compute flow-like distortion
    noiseLacunarity = 2.2;
    noiseOctaves = 12;
    vec3 distort = Fbm3D(twistedPoint * 0.5) * 1.5;
    noiseOctaves = 6;
    float global = (Fbm(twistedPoint + distort) + cloudsCoverage);
    global *= weight;
/**/
    // Compute flow-like distortion
    vec3 p = twistedPoint * cloudsFreq * 6.66;
    vec3 q = p + Fbm3D2(p);
    vec3 r = p + Fbm3D2(q);
    float f = Fbm2(r) * 0.7 + coverage - 0.12;
    float global = saturate(f) * weight;
/**/
 
    // Compute turbilence features
    noiseOctaves = cloudsOctaves;
    float turbulence = Fbm(point * 150.0 * cloudsFreq + Randomize) * 0.0421;
 
    return global;// + turbulence * step(0.1, global);
}
 
//-----------------------------------------------------------------------------
 
void main()
{
    vec3  point  = GetSurfacePoint();
    float height = HeightMapClouds(point);
    OutColor = vec4(height);
}
 
#endif